import 'dart:async';

import 'package:fis_i18n/i18n.dart';
import 'package:fis_measure/interfaces/enums/annotation.dart';
import 'package:fis_measure/interfaces/enums/operate.dart';
import 'package:fis_measure/interfaces/process/player/play_controller.dart';
import 'package:fis_measure/interfaces/process/standard_line/calibration.dart';
import 'package:fis_measure/interfaces/process/workspace/application.dart';
import 'package:fis_measure/interfaces/process/workspace/exam_info.dart';
import 'package:fis_measure/process/workspace/measure_controller.dart';
import 'package:fis_measure/process/workspace/measure_data_controller.dart';
import 'package:fis_measure/process/workspace/measure_handler.dart';
import 'package:fis_measure/process/workspace/third_part/application.dart';
import 'package:fis_measure/view/measure/measure_view_controller.dart';
import 'package:fis_measure/view/menu_button_group/menu_button_group.dart';
import 'package:fis_measure/view/player/controller.dart';
import 'package:fis_measure/view/player/enums.dart';
import 'package:fis_measure/view/player/events.dart';
import 'package:fis_ui/base_define/page.dart';
import 'package:fis_ui/index.dart';
import 'package:fis_ui/widgets/functional/text_tooltip.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:vid/us/vid_us_image.dart';

/// 测量工具 注释工具 切换按钮
class OperateTypeChangeButton extends StatefulWidget implements FWidget {
  const OperateTypeChangeButton({
    super.key,
    required this.businessParent,
  });

  final FPage businessParent;

  @override
  State<OperateTypeChangeButton> createState() =>
      OperateTypeChangeButtonState();
}

// 两个按钮共有三种状态
enum OperateButtonState { isMeasure, isAnnotation, allClose }

class OperateTypeChangeButtonState extends State<OperateTypeChangeButton>
    with TickerProviderStateMixin {
  late final measureHandler = Get.find<MeasureHandler>();
  // late TabController tabController;
  late final measureData = Get.find<MeasureDataController>();
  bool isShowMeasureButton = true;
  bool isWaitingDrawStandardLine = false;
  OperateButtonState operateButtonState = OperateButtonState.allClose;

  /// 全部图片list
  final measureController = Get.find<MeasureController>();
  VidPlayerController playerController =
      Get.find<IPlayerController>() as VidPlayerController;

  bool get isCurrMeasure => operateButtonState == OperateButtonState.isMeasure;
  bool get isCurrAnnotation =>
      operateButtonState == OperateButtonState.isAnnotation;

  @override
  void initState() {
    super.initState();
    measureHandler.annotationsLoaded.addListener(onAnnotationsLoaded);
    measureController.imageLoaded.addListener(_onImageLoaded);
    playerController.eventHandler.addListener(_onHandlePlayerEnvent);
    WidgetsBinding.instance.addPostFrameCallback((timeStamp) {
      handleAutoTapInMeasure();
    });
  }

  @override
  void dispose() {
    super.dispose();
    measureHandler.annotationsLoaded.removeListener(onAnnotationsLoaded);
    measureController.imageLoaded.removeListener(_onImageLoaded);
    playerController.eventHandler.removeListener(_onHandlePlayerEnvent);
    // tabController.dispose();
  }

  void initTab() {
    final application = Get.find<IApplication>();

    ///TODO:[Gavin] 虽然屏蔽了多幅，但是切换至多幅时候仍会存在问题【虽然只显示注释，但是默认会被切换至第一个测量项】
    ///原因是Tab页切换优化为了FOffstage的方式，Measure Tab 实际没有被 dispose
    ///比较妥协的临时解决方案是，触发切换注释时加上 500ms 延时【line 178:180】
    if (application.isSingleFrame && application.visuals.length > 1) {
      //多幅vid
      changeTypeTo(MeasureOperateType.annotation);
      setState(() {
        isShowMeasureButton = false;
        operateButtonState = OperateButtonState.isAnnotation;
      });
    } else {
      if (isWaitingDrawStandardLine) return;

      if (application.checkIs3rdAndEmptyStandardLine()) {
        // 第三方图像参考信息是空的，先关闭测量
        measureHandler.toolPanelState = false;
        measureHandler.canMeasureDrawing = false;
        setState(() {
          operateButtonState = OperateButtonState.allClose;
        });
      } else {
        changeTypeTo(MeasureOperateType.measure);
        measureHandler.toolPanelState = true;
        measureHandler.canMeasureDrawing = true;
        setState(() {
          operateButtonState = OperateButtonState.isMeasure;
        });
      }
    }
  }

  void changeTypeTo(e) {
    measureHandler.currOperateType = e;
    if (e == MeasureOperateType.measure) {
      final measureMetaController = Get.find<MeasureMetaController>();
      measureMetaController.setAvailableModes(measureData.currentMode);
      // 用于切换到第一个测量项
      measureMetaController.updateItemList();
    } else {
      // 用于结束未结束的组合测量项，防止撤销时出现残余
      measureHandler.onRightClickFinishMeasure.emit(this, true);
      final application = Get.find<IApplication>();
      Future.delayed(const Duration(milliseconds: 500), () {
        application.switchAnnotation(AnnotationType.label, "empty");
      });
    }
  }

  // 应该在图像注释获取完成后再触发initTab
  void onAnnotationsLoaded(Object sender, _) {
    if (!mounted) return;
    initTab();
  }

  void _onFrameLoadedAfterFirst(Object sender, VidUsImage? e) {
    if (!mounted) return;
    initTab();
    playerController.firstFrameLoaded.removeListener(_onFrameLoadedAfterFirst);
  }

  void _onImageLoaded(Object sender, ExamImageInfo? e) {
    playerController = Get.find<IPlayerController>() as VidPlayerController;
    playerController.firstFrameLoaded.addListener(_onFrameLoadedAfterFirst);
  }

  /// 处理自动进入测量状态
  void handleAutoTapInMeasure() {
    if (isShowMeasureButton == false) {
      enterAnnotation();
      return;
    }
    final application = Get.find<IApplication>();
    bool canEnterMeasure = true;
    if (application.checkIs3rdAndEmptyStandardLine() &&
        application.isSingleFrame == false) {
      // 校准信息为空
      final thirdPartApp = (application as ThirdPartApplication);
      if (thirdPartApp.hasFillEmtpyStandardLineRequested) {
        // 不是首次请求，不再自动进入测量
        canEnterMeasure = false;
      }
    }
    if (canEnterMeasure) {
      enterMeasure();
    }
  }

  /// 处理自动退出测量状态
  void handleAutoExitMeasure() {
    if (Get.find<IApplication>().checkIs3rdAndEmptyStandardLine()) {
      closeAll();
    } else {
      enterMeasure();
    }
  }

  @override
  void didUpdateWidget(covariant OperateTypeChangeButton oldWidget) {
    super.didUpdateWidget(oldWidget);
  }

  onTapMeasure() {
    if (operateButtonState == OperateButtonState.isMeasure) {
      closeAll();
    } else {
      enterMeasure();
    }
  }

  onTapAnnotation() {
    if (operateButtonState == OperateButtonState.isAnnotation) {
      closeAll();
    } else {
      enterAnnotation();
    }
  }

  /// 进入注释
  void enterAnnotation() {
    measureHandler.toolPanelState = true;
    measureHandler.canMeasureDrawing = true;
    changeTypeTo(MeasureOperateType.annotation);
    setState(() {
      operateButtonState = OperateButtonState.isAnnotation;
    });
  }

  /// 进入测量
  void enterMeasure() {
    final application = Get.find<IApplication>();
    if (application.checkIs3rdAndEmptyStandardLine()) {
      VoidCallback rollbackFn;
      if (operateButtonState == OperateButtonState.isAnnotation) {
        rollbackFn = onTapAnnotation;
      } else {
        rollbackFn = closeAll;
      }

      final thirdPartApp = (application as ThirdPartApplication);
      thirdPartApp.hasFillEmtpyStandardLineRequested = true;

      _tryEnterMeasure(rollbackFn);
    } else {
      _tryEnterMeasure(() {});
    }
  }

  closeAll() {
    measureHandler.toolPanelState = false;
    measureHandler.canMeasureDrawing = false;
    setState(() {
      operateButtonState = OperateButtonState.allClose;
    });
  }

  /// 进入测量
  Future<void> _tryEnterMeasure(VoidCallback rejectCallback) async {
    isWaitingDrawStandardLine = true;
    _setEnterMeasureState(); // 预进入

    final checked = await _checkMeasureCanEnter();
    if (checked) {
      _setEnterMeasureState();
    } else {
      rejectCallback();
    }
    isWaitingDrawStandardLine = false;
  }

  /// 检查是否可进入测量
  Future<bool> _checkMeasureCanEnter() {
    final application = Get.find<IApplication>();
    if (application.checkIs3rdAndEmptyStandardLine()) {
      // 第三方图像 像素比信息空，需要通过校准线校准
      return _waitStandardLineDone();
    } else {
      return Future.value(true);
    }
  }

  /// 等待校准线完成
  Future<bool> _waitStandardLineDone() {
    final completer = Completer<bool>();
    FConfirmAlert.show(
      context: context,
      confrimLabel: i18nBook.common.confirm.t,
      cancelLabel: i18nBook.common.cancel.t,
      title: i18nBook.common.tip.t,
      subTitle: i18nBook.measure.thirdImageNeedCalibratingTips.t,
      businessParent: widget.businessParent,
      onConfirm: () {
        final calibrationController =
            Get.find<IStandardLineCalibrationController>();

        void onEditStateChanged(_, StandardLineCalibrationEditState e) {
          void finish(bool result) {
            // 移除监听
            calibrationController.editStateChanged
                .removeListener(onEditStateChanged);
            // 返回异步结果
            completer.complete(result);
          }

          if (e == StandardLineCalibrationEditState.done) {
            finish(true);
          } else if (e == StandardLineCalibrationEditState.inactive) {
            finish(false);
          } else {
            // no process
          }
        }

        // 开始监听
        calibrationController.editStateChanged.addListener(onEditStateChanged);

        // 进入编辑
        calibrationController.enterEditMode();
      },
      onCancel: () {
        completer.complete(false);
      },
    );
    return completer.future;
  }

  /// 生成状态回滚函数
  VoidCallback _generateStatesRollbackFn() {
    final toolPanelStateSnapshot = measureHandler.toolPanelState;
    final canMeasureDrawingSnapshot = measureHandler.canMeasureDrawing;
    final operateButtonStateSnapshot = operateButtonState;
    final operateTypeSnapshot = measureHandler.currOperateType;
    void fn() {
      measureHandler.toolPanelState = toolPanelStateSnapshot;
      measureHandler.canMeasureDrawing = canMeasureDrawingSnapshot;
      changeTypeTo(operateTypeSnapshot);
      setState(() {
        operateButtonState = operateButtonStateSnapshot;
      });
    }

    return fn;
  }

// 进入测量状态
  void _setEnterMeasureState([bool showToolPanel = true]) {
    if (measureHandler.toolPanelState != showToolPanel) {
      measureHandler.toolPanelState = showToolPanel;
    }
    if (measureHandler.canMeasureDrawing == false) {
      measureHandler.canMeasureDrawing = true;
    }
    if (measureHandler.currOperateType != MeasureOperateType.measure) {
      changeTypeTo(MeasureOperateType.measure);
    }
    if (operateButtonState != OperateButtonState.isMeasure) {
      setState(() {
        isShowMeasureButton = true;
        operateButtonState = OperateButtonState.isMeasure;
      });
    }
  }

  void _onHandlePlayerEnvent(Object sender, VidPlayerEvent e) {
    if (e is VidPlayerStatusChangeEvent) {
      if (e.status == VidPlayStatus.play) {
        handleAutoExitMeasure();
      }
    }
  }

  @override
  FWidget build(BuildContext context) {
    return FSizedBox(
        width: 60,
        child: FColumn(children: [
          if (isShowMeasureButton)
            FTextTooltip(
              position: DisplayPosition.left,
              textStyle: const TextStyle(fontSize: 16, color: Colors.white),
              height: 36,
              message: i18nBook.measure.measureTool.t,
              margin: const EdgeInsets.only(right: 10),
              decoration: BoxDecoration(
                color: Colors.black.withOpacity(1.0),
                borderRadius: BorderRadius.circular(4),
              ),
              child: FInkWell(
                onTap: onTapMeasure,
                child: FContainer(
                  height: 34,
                  margin: const EdgeInsets.only(
                    top: 5,
                  ),
                  padding:
                      const EdgeInsets.symmetric(horizontal: 10, vertical: 5),
                  decoration: BoxDecoration(
                    color: isCurrMeasure ? Colors.grey : Colors.transparent,
                    borderRadius: BorderRadius.circular(5),
                  ),
                  child: FIcon(
                    Icons.straighten,
                    color: isCurrMeasure ? Colors.white : Colors.white,
                  ),
                ),
              ),
            ),
          FTextTooltip(
            position: DisplayPosition.left,
            textStyle: const TextStyle(fontSize: 16, color: Colors.white),
            height: 36,
            message: i18nBook.measure.annotationTool.t,
            margin: const EdgeInsets.only(right: 10),
            decoration: BoxDecoration(
              color: Colors.black.withOpacity(1.0),
              borderRadius: BorderRadius.circular(4),
            ),
            child: FInkWell(
              onTap: onTapAnnotation,
              child: FContainer(
                height: 34,
                margin: const EdgeInsets.only(
                  top: 5,
                ),
                padding:
                    const EdgeInsets.symmetric(horizontal: 10, vertical: 5),
                decoration: BoxDecoration(
                  color: isCurrAnnotation ? Colors.grey : Colors.transparent,
                  borderRadius: BorderRadius.circular(5),
                ),
                child: FIcon(
                  Icons.font_download,
                  color: isCurrAnnotation ? Colors.white : Colors.white,
                ),
              ),
            ),
          )
        ]));
  }
}
