import 'package:fis_i18n/i18n.dart';
import 'package:fis_measure/interfaces/process/workspace/measure_3d_view_controller.dart';
import 'package:fis_measure/process/workspace/measure_3d_view_controller.dart';
import 'package:fis_measure/process/workspace/measure_controller.dart';
import 'package:fis_ui/index.dart';
import 'package:flutter/material.dart';
import '../controller.dart';
import 'package:fis_measure/interfaces/process/player/play_controller.dart';
import 'package:fis_measure/view/mobile_view/widgets/throttle.dart' as utils;
import 'package:get/get.dart';
import '../events.dart';

part 'tone_bar.dart';
part 'tone_bar_carotid.dart';

class OperateBar extends StatefulWidget implements FWidget {
  const OperateBar({Key? key, this.ifShowResetBtn = true}) : super(key: key);
  final bool ifShowResetBtn;

  @override
  State<OperateBar> createState() => _OperateBar();
}

class _OperateBar extends State<OperateBar> {
  final measure3DViewController = Get.find<Measure3DViewController>();
  bool enableCarotidBar = false;
  double get devicePixelRatio =>
      kIsMobile ? 1 : MediaQuery.of(context).devicePixelRatio;
  @override
  void initState() {
    measure3DViewController.updatePlayerMode.addListener(_onModeChanged);
    super.initState();
  }

  @override
  void dispose() {
    measure3DViewController.updatePlayerMode.removeListener(_onModeChanged);
    super.dispose();
  }

  /// 模式改变触发更新
  void _onModeChanged(Object s, MeasureMode mode) {
    switch (mode) {
      case MeasureMode.vidMode:
        setState(() {
          enableCarotidBar = false;
        });
        break;
      case MeasureMode.carotid2DMode:
        setState(() {
          enableCarotidBar = true;
        });
        break;
      case MeasureMode.carotid3DMode:
        setState(() {
          enableCarotidBar = true;
        });
        break;
    }
  }

  @override
  Widget build(BuildContext context) {
    return SizedBox(
      width: 300 / devicePixelRatio,
      child: Center(
        child:
            enableCarotidBar ? _buildCarotidOperateBar() : _buildOperateBar(),
      ),
    );
  }

  Widget _buildOperateBar() {
    return Column(
      mainAxisAlignment: MainAxisAlignment.center,
      children: [
        _BrightnessToneBar(),
        SizedBox(height: 10 / devicePixelRatio),
        _ContrastToneBar(),
        if (widget.ifShowResetBtn) ...[
          SizedBox(height: 10 / devicePixelRatio),
          _ResetToneButton(),
          SizedBox(height: 10 / devicePixelRatio),
        ]
      ],
    );
  }

  Widget _buildCarotidOperateBar() {
    return Column(
      mainAxisAlignment: MainAxisAlignment.center,
      children: [
        _SharpnessCarotidToneBar(),
        SizedBox(height: 10 / devicePixelRatio),
        _BrightnessCarotidToneBar(),
        SizedBox(height: 10 / devicePixelRatio),
        _ContrastCarotidToneBar(),
        SizedBox(height: 10 / devicePixelRatio),
        _ResetCarotidToneButton(),
        SizedBox(height: 10 / devicePixelRatio),
      ],
    );
  }
}
